@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.test;
 
 /*
- * Copyright (c) 1999, 2017 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2018 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -33,6 +33,7 @@ import java.io.BufferedReader;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.IOException;
+import java.io.UnsupportedEncodingException;
 import java.util.Enumeration;
 import java.util.Properties;
 
@@ -95,7 +96,7 @@ public abstract class AbstractActionApp
      * Constructors
      *-------------------------------------------------------------*/
     protected AbstractActionApp() {
-        m_runner = new Thread( Main.getRes().getString( "WrapperActionTest_Runner" ) )
+        m_runner = new Thread( "WrapperActionTest_Runner" )
         {
             public void run()
             {
@@ -191,9 +192,33 @@ public abstract class AbstractActionApp
     
     protected void prepareSystemOutErr()
     {
-        m_out = new DeadlockPrintStream( System.out );
+        boolean streamsSet = false;
+        if ( "true".equals( System.getProperty( "wrapper.use_sun_encoding" ) ) ) {
+            String sunStdoutEncoding = System.getProperty( "sun.stdout.encoding" );
+            if ( ( sunStdoutEncoding != null ) && !sunStdoutEncoding.equals( System.getProperty( "file.encoding" ) ) ) {
+                /* We need to create the stream using the same encoding as the one used for stdout, else this will lead to encoding issues. */
+                try
+                {
+                    m_out = new DeadlockPrintStream( System.out, false, sunStdoutEncoding );
+                    m_err = new DeadlockPrintStream( System.err, false, sunStdoutEncoding );
+                    streamsSet = true;
+                }
+                catch ( UnsupportedEncodingException e )
+                {
+                    /* This should not happen because we always make sure the encoding exists before launching a JVM.
+                     *  If any of the above streams failed, we want to fall back to streams that use the same encoding.
+                     *  This message can be localized because we did not redirect the streams yet. */
+                    System.out.println( Main.getRes().getString( "Failed to set the encoding '{0}' when creating a DeadlockPrintStream.\n Make sure the value of sun.stdout.encoding is correct.", sunStdoutEncoding ) );
+                }
+            }
+        }
+        if ( !streamsSet )
+        {
+            m_out = new DeadlockPrintStream( System.out );
+            m_err = new DeadlockPrintStream( System.err );
+        }
+        
         System.setOut( m_out );
-        m_err = new DeadlockPrintStream( System.err );
         System.setErr( m_err );
     }
     
