@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.test;
 
 /*
- * Copyright (c) 1999, 2011 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2018 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -34,7 +34,7 @@ import org.tanukisoftware.wrapper.WrapperManager;
 /**
  *
  *
- * @author Leif Mortenson <leif@tanukisoftware.com>
+ * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
  */
 public class LogOutput {
     private static void sleep() {
@@ -90,6 +90,10 @@ public class LogOutput {
         for (int i = 0; i < 100; i++) {
             WrapperManager.log(WrapperManager.WRAPPER_LOG_LEVEL_INFO, sa);
         }
+        
+        /* This string should not be translated because it contains Japanese characters. 
+         *  A conversion error will be raised if a key of a translation file contains non-ascii characters. */
+        WrapperManager.log(WrapperManager.WRAPPER_LOG_LEVEL_INFO, "Japanese \u300c\u65e5\u672c\u8a9e\u306e\u8a66\u9a13 \u30a2\u30a4\u30a6\u30a8\u30aa\u30ab\u30ad\u30af\u30b1\u30b3\u300d Test");
     }
 }
 
