@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.test;
 
 /*
- * Copyright (c) 1999, 2016 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2018 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -34,6 +34,7 @@ import java.awt.Button;
 import java.awt.Checkbox;
 import java.awt.Component;
 import java.awt.Container;
+import java.awt.Font;
 import java.awt.Frame;
 import java.awt.GridBagConstraints;
 import java.awt.GridBagLayout;
@@ -46,6 +47,13 @@ import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.WindowEvent;
 import java.awt.event.WindowListener;
+import javax.swing.JLabel;
+import javax.swing.JPanel;
+import javax.swing.JDialog;
+import javax.swing.BoxLayout;
+import javax.swing.border.Border;
+import javax.swing.border.EmptyBorder;
+import javax.swing.border.CompoundBorder;
 import org.tanukisoftware.wrapper.WrapperActionServer;
 import org.tanukisoftware.wrapper.WrapperListener;
 import org.tanukisoftware.wrapper.WrapperManager;
@@ -53,6 +61,8 @@ import org.tanukisoftware.wrapper.WrapperResources;
 import org.tanukisoftware.wrapper.WrapperSystemPropertyUtil;
 import org.tanukisoftware.wrapper.event.WrapperEventListener;
 import org.tanukisoftware.wrapper.event.WrapperSecondInvocationEvent;
+import org.tanukisoftware.wrapper.event.WrapperServicePauseEvent;
+import org.tanukisoftware.wrapper.event.WrapperServiceResumeEvent;
 import org.tanukisoftware.wrapper.event.WrapperEvent;
 
 /**
@@ -67,7 +77,7 @@ import org.tanukisoftware.wrapper.event.WrapperEvent;
  *  see the <a href="http://wrapper.tanukisoftware.com/doc/english/integrate.html">integration</a>
  *  section of the documentation for more details.
  *
- * @author Leif Mortenson <leif@tanukisoftware.com>
+ * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
  */
 public class Main
     extends AbstractActionApp
@@ -86,6 +96,8 @@ public class Main
     private TextField m_childCommand;
     private Checkbox m_childDetached;
     
+    private JDialog m_pauseDialog;
+    
     /*---------------------------------------------------------------
      * Constructors
      *-------------------------------------------------------------*/
@@ -101,15 +113,26 @@ public class Main
          * Serial Version UID.
          */
         private static final long serialVersionUID = -3847376282833547574L;
+        private float dpiScaleFactor = 1;
 
         MainFrame()
         {
             super( getRes().getString( "TestWrapper Example Application" ) );
             
+            try
+            {
+                dpiScaleFactor = (float)WrapperManager.nativeGetDpiScale() / 96;
+            }
+            catch ( UnsatisfiedLinkError e )
+            {
+                // This can happen if an old native library is used.  Fall through and make sure the scale factor is 1.
+                dpiScaleFactor = 1;
+            }
+            
             init();
             
             setLocation( 10, 10 );
-            setSize( 750, 480 );
+            setSize( (int)(750*dpiScaleFactor), (int)(480*dpiScaleFactor));
             
             setResizable( true );
         }
@@ -184,7 +207,7 @@ public class Main
             buildCommand( panel, gridBag, c, getRes().getString( "Simulate Slow JVM" ), "appear_slow", slowPanel2 );
             
             buildCommand( panel, gridBag, c, getRes().getString( "Create Deadlock" ), "deadlock",
-                    getRes().getString( "Creates two new threads which intentionally go into a DeadLock situation.  (Standard, Professional)" ) );
+                    getRes().getString( "Creates two new threads which intentionally go into a deadlock situation.  (Standard, Professional)" ) );
             
             buildCommand( panel, gridBag, c, getRes().getString("Simulate Out Of Memory" ), "outofmemory",
                     getRes().getString( "Throws an OutOfMemoryError to demonstrate the Trigger feature." ) );
@@ -202,7 +225,7 @@ public class Main
                     getRes().getString( "Begins calling WrapperManager.getUser() and getInteractiveUser() to monitor the current and interactive users." ) );
             
             buildCommand( panel, gridBag, c, getRes().getString( "Poll Users with Groups" ), "groups",
-                    getRes().getString( "Same as above, but includes information about the user's groups." ) );
+                    getRes().getString( "Same as above, but includes information about the user''s groups." ) );
             
             buildCommand( panel, gridBag, c, getRes().getString( "Console" ), "console", getRes().getString( "Prompt for Actions in the console." ) );
             
@@ -220,6 +243,7 @@ public class Main
             m_listenerFlags.add( "Control" );
             m_listenerFlags.add( "Logging" );
             m_listenerFlags.add( "Core" );
+            m_listenerFlags.add( "RemoteControl" );
             
             Panel flagPanel = new Panel();
             flagPanel.setLayout( new BorderLayout() );
@@ -280,7 +304,10 @@ public class Main
             buildCommand( panel, gridBag, c, getRes().getString( "Is Standard?" ), "is_standard", getRes().getString( "Prints true if this is a Standard Edition." ) );
             
             addWindowListener( this );
-            WrapperManager.addWrapperEventListener( Main.this, WrapperEventListener.EVENT_FLAG_REMOTE_CONTROL );
+            
+            // By default listen to Service and RemoteControl events.
+            buildEventMask( new String[] { "Service", "RemoteControl" } );
+            updateEventListener();
         }
         
         private void buildCommand( Container container,
@@ -292,6 +319,9 @@ public class Main
         {
             Button button = new Button( label );
             button.setActionCommand( command );
+            if (dpiScaleFactor != 1) {
+                button.setFont(new Font("Arial", Font.PLAIN, (int)(12 * dpiScaleFactor)));
+            }
             
             c.fill = GridBagConstraints.BOTH;
             c.gridwidth = 1;
@@ -313,11 +343,46 @@ public class Main
             {
                 desc = new Label( description.toString() );
             }
+            if (dpiScaleFactor != 1) {
+                desc.setFont(new Font("Arial", Font.PLAIN, (int)(12 * dpiScaleFactor)));
+            }
             
             gridBag.setConstraints( desc, c );
             container.add( desc );
         }
         
+        private void buildEventMask(String[] flags) 
+        {
+            // Create the mask.
+            long mask = 0;
+            for ( int i = 0; i < flags.length; i++ )
+            {
+                String flag = flags[i];
+                if ( flag.equals( "Service" ) )
+                {
+                    mask |= WrapperEventListener.EVENT_FLAG_SERVICE;
+                }
+                else if ( flag.equals( "Control" ) )
+                {
+                    mask |= WrapperEventListener.EVENT_FLAG_CONTROL;
+                }
+                else if ( flag.equals( "Logging" ) )
+                {
+                    mask |= WrapperEventListener.EVENT_FLAG_LOGGING;
+                }
+                else if ( flag.equals( "Core" ) )
+                {
+                    mask |= WrapperEventListener.EVENT_FLAG_CORE;
+                }
+                else if ( flag.equals( "RemoteControl" ) )
+                {
+                    mask |= WrapperEventListener.EVENT_FLAG_REMOTE_CONTROL;
+                }
+            }
+            
+            setEventMask( mask );
+        }
+        
         /**************************************************************************
          * ActionListener Methods
          *************************************************************************/
@@ -326,31 +391,8 @@ public class Main
             String action = event.getActionCommand();
             if ( action.equals( "listener" ) )
             {
-                // Create the mask.
-                long mask = 0;
                 String[] flags = m_listenerFlags.getSelectedItems();
-                for ( int i = 0; i < flags.length; i++ )
-                {
-                    String flag = flags[i];
-                    if ( flag.equals( "Service" ) )
-                    {
-                        mask |= WrapperEventListener.EVENT_FLAG_SERVICE;
-                    }
-                    else if ( flag.equals( "Control" ) )
-                    {
-                        mask |= WrapperEventListener.EVENT_FLAG_CONTROL;
-                    }
-                    else if ( flag.equals( "Logging" ) )
-                    {
-                        mask |= WrapperEventListener.EVENT_FLAG_LOGGING;
-                    }
-                    else if ( flag.equals( "Core" ) )
-                    {
-                        mask |= WrapperEventListener.EVENT_FLAG_CORE;
-                    }
-                }
-                
-                setEventMask( mask );
+                buildEventMask(flags);
             }
             
             int slowSeconds;
@@ -464,6 +506,16 @@ public class Main
             // bring the window to front
             bringToFront(m_frame);
         }
+        if (event instanceof WrapperServicePauseEvent )
+        {
+            // pause the demo application
+            pause();
+        }
+        if (event instanceof WrapperServiceResumeEvent )
+        {
+            // resume the demo application
+            resume();
+        }
     }
 
     /*---------------------------------------------------------------
@@ -514,14 +566,10 @@ public class Main
             System.out.println( getRes().getString( "TestWrapper:     U: Unexpected Halt (Simulate crash)" ) );
             System.out.println( getRes().getString( "TestWrapper:     V: Access Violation (Actual crash)" ) );
             System.out.println( getRes().getString( "TestWrapper:     G: Make the JVM appear to be hung." ) );
-            System.out.println( "TestWrapper:" );
         }
         catch ( java.io.IOException e )
         {
             System.out.println( getRes().getString( "TestWrapper: Unable to open the action server socket: {0}", e.getMessage() ) );
-            System.out.println( "TestWrapper:" );
-            m_actionServer = null;
-            System.out.println( "TestWrapper:" );
             m_actionServer = null;
         }
         
@@ -617,6 +665,66 @@ public class Main
         return exitCode;
     }
     
+    /**
+     * Pause the demo application. 
+     */
+    private void pause() {
+        // Update the title with the "paused" state.
+        m_frame.setTitle( m_frame.getTitle() + getRes().getString( " (paused)" ) );
+        
+        // Disable all components of the frame.
+        enableComponents( m_frame, false );
+        
+        // Show a dialog centered over the frame to indicate that the application was paused and is waiting the resume event.
+        JLabel label1 = new JLabel();
+        label1.setHorizontalAlignment( javax.swing.SwingConstants.CENTER );
+        label1.setAlignmentX(Component.CENTER_ALIGNMENT);
+        label1.setText( getRes().getString( "The demo application is paused." ) );
+        
+        JLabel label2 = new JLabel();
+        label2.setHorizontalAlignment( javax.swing.SwingConstants.CENTER );
+        label2.setAlignmentX(Component.CENTER_ALIGNMENT);
+        label2.setText( getRes().getString( "Resume it to make this dialog box disappear." ) );
+      
+        Border border1 = label1.getBorder();
+        Border margin1 = new EmptyBorder( 25, 25, 0, 25 );
+        label1.setBorder( new CompoundBorder(border1, margin1) );
+      
+        Border border2 = label2.getBorder();
+        Border margin2 = new EmptyBorder( 0, 25, 30, 25 );
+        label2.setBorder( new CompoundBorder(border2, margin2) );
+        
+        JPanel panel = new JPanel();
+        panel.setLayout(new BoxLayout(panel, BoxLayout.PAGE_AXIS));
+        panel.add( label1 );
+        panel.add( label2 );
+
+        m_pauseDialog = new JDialog( m_frame, m_frame.getTitle(), false );
+        m_pauseDialog.getContentPane().add( panel );
+        m_pauseDialog.setResizable( false );
+        m_pauseDialog.setDefaultCloseOperation( javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE );
+        m_pauseDialog.pack();
+        m_pauseDialog.setLocationRelativeTo( m_frame );
+        m_pauseDialog.setVisible( true );
+    }
+
+    /**
+     * Resume the application. 
+     */
+    private void resume() {
+        String title = m_frame.getTitle();
+        // Update the title to remove the "paused" state.
+        m_frame.setTitle( title.substring( 0, title.indexOf( getRes().getString( " (paused)" ) ) ) );
+
+        // Enable all components of the frame.
+        enableComponents( m_frame, true );
+        
+        // Close the dialog.
+        m_pauseDialog.setDefaultCloseOperation( javax.swing.WindowConstants.DISPOSE_ON_CLOSE );
+        m_pauseDialog.setVisible(false);
+        m_pauseDialog.dispatchEvent(new WindowEvent(m_pauseDialog, WindowEvent.WINDOW_CLOSING));
+    }
+    
     public void controlEvent( int event )
     {
         System.out.println( getRes().getString( "TestWrapper: controlEvent({0})",  new Integer( event ) ) );
@@ -718,6 +826,19 @@ public class Main
         }
     }
     
+    /**
+     * Helper method to disable or enable all components of a container
+     */
+    private void enableComponents(Container container, boolean enable) {
+        Component[] components = container.getComponents();
+        for (int a = 0; a < components.length; a++) {
+            components[a].setEnabled( enable );
+            if (components[a] instanceof Container) {
+                enableComponents((Container)components[a], enable);
+            }
+        }
+    }
+    
     /*---------------------------------------------------------------
      * Main Method
      *-------------------------------------------------------------*/
